package common;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashMap;

import common.StreamIdentifier.StreamAnalyzerI;
import common.avc.AvcConstants;
import common.mpeg.MpegConstants;
import common.mpeg.MpegUtil;

public class AvcVideoAnalyzer implements StreamAnalyzerI, AvcConstants {
	private static final HashMap<Byte, String> profileNameMap=new HashMap<Byte, String>();
	private int lastNalUnitType=NAL_UNIT_ACCESS_DELIMETER;
	private boolean cpbDpbDelaysPresentFlag=false;
	private int cpbRemovalDelayLengthMinus1;
	private int dpbOutputDelayLengthMinus1;

	static {
		profileNameMap.put(Byte.valueOf(BASELINE_PROFILE), BASELINE_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(MAIN_PROFILE), MAIN_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(EXTENDED_PROFILE), EXTENDED_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(HIGH_PROFILE1), HIGH_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(HIGH_PROFILE2), HIGH_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(HIGH_PROFILE3), HIGH_PROFILE_NAME);
		profileNameMap.put(Byte.valueOf(HIGH_PROFILE4), HIGH_PROFILE_NAME);
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ByteBuffer buffer=ByteBuffer.allocate(1024);
		try {
			FileChannel fc=new RandomAccessFile("m:/beowulf/Beowulf_e2.264","r").getChannel();
			fc.read(buffer);
			buffer.rewind();
			//buffer.position(7);
			BitStream bitStream=new BitStream(buffer);
			System.out.println(new AvcVideoAnalyzer().analyze(bitStream, 0,0));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	public StreamDescriptor analyze(MuxBuffer packetSource, int stream, int offset) { 
		ByteBuffer buffer=packetSource.getByteBuffer();
		BitStream bitStream=(packetSource instanceof BitStream)?((BitStream)packetSource):new BitStream(buffer);
		bitStream.setPosition(offset);
		
		int startCode;
		VideoStreamDescriptor sd=null;
		while ((startCode=MpegUtil.findNextStartCode(buffer)) > 0) {
			int nal=buffer.getInt();
			//Make sure we are on a NAL startCode
			if ((nal & ((int)0xffffff00))==0x00000100) {
				lastNalUnitType=nal & 0x1f;
				if (lastNalUnitType==7) {
					sd=getStreamDescriptor(stream);
					parseSequenceParameters(getEscapedBitStream(bitStream), sd);
				} else if (lastNalUnitType != NAL_UNIT_ACCESS_DELIMETER) {
					break;
				}
			}
			buffer.position(startCode);
		}
		return sd;
	}
	private BitStream getEscapedBitStream(BitStream inBitStream) {
		ByteBuffer in=inBitStream.getByteBuffer();
		int size=Math.min(0x80, in.remaining());
		ByteBuffer out=ByteBuffer.allocate(size);
		int value=in.getShort() & 0xffff;
		size-=2;
		for (int i=0;i<size;i++) {
			value=((value << 8) + (in.get() & 0xff)) & 0xffffff;
			if (value==0x3) {
				value = (value & 0xffff00) + (in.get() & 0xff);
			}
			out.put((byte)(value >> 16));
		}
		out.putShort((short)value);
		out.position(0);
		BitStream outBitStream=new BitStream(out);
		return outBitStream;
	}
	
	public void pareseSei(BitStream bitStream, int end, VideoStreamDescriptor sd) {
		ByteBuffer buffer=bitStream.getByteBuffer();
		while (buffer.position() < end) {
			parseSeiMessage(bitStream, sd);
			
		}
	}
	private void parseSeiMessage(BitStream bitStream, VideoStreamDescriptor sd) {
		ByteBuffer buffer=bitStream.getByteBuffer();
		int payloadType=0;
		
		byte b;
		while ((b=buffer.get())==(byte)0xff) {
			payloadType +=0xff;
		}
		payloadType += b & 0xff;
		
		int payloadLen=0;
		while ((b=buffer.get())==(byte)0xff) {
			payloadLen +=0xff;
		}
		payloadLen += b & 0xff;
		int pos=buffer.position()+payloadLen;
		if (payloadType==1) {
			parseSeiMessagePicTiming(bitStream, sd);
		} 
		buffer.position(pos);
		
	}
	
	private void parseSeiMessagePicTiming(BitStream bitStream, VideoStreamDescriptor sd) {
		if (cpbDpbDelaysPresentFlag) {
			int cpb_removal_delay=bitStream.getBits(this.cpbRemovalDelayLengthMinus1+1);
			int dpb_output_delay=bitStream.getBits(this.dpbOutputDelayLengthMinus1+1);
		}
		
	}
	public VideoStreamDescriptor getStreamDescriptor(int streamId) {
		VideoStreamDescriptor sd=new VideoStreamDescriptor(streamId);
		sd.setFileNameExtension(FILE_EXTENSION);
		sd.setTsType(MpegConstants.TS_STREAM_TYPE_AVC_VIDEO);
		return sd;
	}
	public void parseSequenceParameters(BitStream bitStream, VideoStreamDescriptor sd) {

		byte profileIdc=bitStream.get();
		byte flags=bitStream.get(); //4 flags + 4 zeros
		byte levelIdc=bitStream.get();
		sd.setCodec(CODEC_NAME);
		sd.setProfileName(profileNameMap.get(Byte.valueOf(profileIdc)));
		sd.setLevelName(levelIdc/10 + "." + levelIdc%10);
		int seqParameterSetId=bitStream.getExpGolombCode();
		
		if (profileIdc==HIGH_PROFILE1 || profileIdc==HIGH_PROFILE2 || profileIdc==HIGH_PROFILE3 || profileIdc==HIGH_PROFILE4) {
			int chromaFormatIdc=bitStream.getExpGolombCode();
			if (chromaFormatIdc==3) {
				int residualColourTransformFlag=bitStream.getBit();
			}
			int bitDepthLumaMinus8=bitStream.getExpGolombCode();
			int bitDepthChromaMinus8=bitStream.getExpGolombCode();
			int qpprimeYZeroTransformBypassFlag=bitStream.getBit();
			int seqScalingMatrixPresentFlag=bitStream.getBit();
			if (seqScalingMatrixPresentFlag==1) {
				for (int i=0;i<8;i++) {
					int seqScalingListPresentFlag=bitStream.getBit();
					if (seqScalingListPresentFlag==1) {
						parseScalingList(bitStream,i<6?16:64);
					}
				}
				//byte seqScalingMatrixPresentFlags=bitStream.getByte();
			}
			int log2MaxFrameNumMinus4=bitStream.getExpGolombCode();
			int picOrderCntType=bitStream.getExpGolombCode();
			if (picOrderCntType==0) {
				int log2MaxPicOrderCntLsbMinus4=bitStream.getExpGolombCode();
			} else if (picOrderCntType==1) {
				int deltaPicOrderAlwaysZeroFlag=bitStream.getBit();
				int offsetForNonRefPic=bitStream.getExpGolombCode();//TODO: Signed
				int offsetForTopToBottomField=bitStream.getExpGolombCode();//TODO: Signed
				int numRefFramesInPicOrderCntCycle=bitStream.getExpGolombCode();
				for (int i=0;i<numRefFramesInPicOrderCntCycle;i++) {
					bitStream.getExpGolombCode();//TODO: Signed
				}
			}
			int numRefFrames=bitStream.getExpGolombCode();
			int gapsInFrameNumValueAllowedFlag=bitStream.getBit();
			int picWidthInMbsMinus1=bitStream.getExpGolombCode();
			int picHeightInMapUnitsMinus1=bitStream.getExpGolombCode();
			int frameMbsOnlyFlag=bitStream.getBit();
			//Derived this all on my lonesome, not 100% sure it is accurate
			sd.setInterlaced(1-frameMbsOnlyFlag);
			if (frameMbsOnlyFlag==0) {
				bitStream.getBit();//mb_adaptive_frame_field_flag
			}
			int direct8x8InferenceFlag=bitStream.getBit();
			int frameCroppingFlag=bitStream.getBit();
			int frameCropLeftOffset=0,frameCropRightOffset=0,frameCropTopOffset=0,frameCropBottomOffset=0;
			if (frameCroppingFlag==1) {
				frameCropLeftOffset=bitStream.getExpGolombCode();
				frameCropRightOffset=bitStream.getExpGolombCode();
				frameCropTopOffset=bitStream.getExpGolombCode();
				frameCropBottomOffset=bitStream.getExpGolombCode();
			}
			//Borrowed heavily from 
		    int width =(picWidthInMbsMinus1       +1)*16;
		    int height=(picHeightInMapUnitsMinus1+1)*16*(2-frameMbsOnlyFlag);
		    int cropUnitX=SUB_WIDTH_C[chromaFormatIdc];
		    int cropUnitY=SUB_HEIGHT_C[chromaFormatIdc]*(2-frameMbsOnlyFlag);
		    width -=(frameCropLeftOffset+frameCropRightOffset )*cropUnitX;
		    height-=(frameCropTopOffset +frameCropBottomOffset)*cropUnitY;
			sd.setWidth(width);
			sd.setHeight(height);
			int vuiParametersPresentFlag=bitStream.getBit();
			
			if (vuiParametersPresentFlag==1) {
				int aspectRatioIdc=-1;
				int aspectRatioInfoPresentFlag=bitStream.getBit();
				if (aspectRatioInfoPresentFlag==1) {
					aspectRatioIdc=bitStream.getBits(8);
					if (aspectRatioIdc==EXTENDED_SAR) {
						sd.setPixelAspectRatio(new int[] {bitStream.getBits(16),bitStream.getBits(16)});
					} else {
						sd.setPixelAspectRatio(ASPECT_RATIO[aspectRatioIdc]);
					}
				}
				
				int overscanInfoPresentFlag=bitStream.getBit();
				if (overscanInfoPresentFlag==1) {
					bitStream.getBit(); // overscan_appropriate_flag
				}
				int videoSignalTypePresentFlag=bitStream.getBit();
				if (videoSignalTypePresentFlag==1) {
					int videoFormat=bitStream.getBits(3);
					int videoFullRangeFlag=bitStream.getBit();
					int colourDescriptionPresentFlag=bitStream.getBit();
					if (colourDescriptionPresentFlag==1) {
						byte colourPrimaries=bitStream.getByte();
						byte transferCharacteristics=bitStream.getByte();
						byte matrixCoefficients=bitStream.getByte();
					}
				}
				int chromaLocInfoPresentFlag=bitStream.getBit();
				if (chromaLocInfoPresentFlag==1) {
					bitStream.getExpGolombCode();//chroma_sample_loc_type_top_field
					bitStream.getExpGolombCode();//chroma_sample_loc_type_bottom_field
				}
				//F___ing finally
				int timingInfoPresentFlag=bitStream.getBit();
				if (timingInfoPresentFlag==1) {
					int numUnitsInTick=(bitStream.getBits(16) << 16) + bitStream.getBits(16);
					int timeScale=(bitStream.getBits(16) << 16) + bitStream.getBits(16);
					int fixedFrameRateFlag=bitStream.getBit();
					sd.setFrameRateRatio(new int[]{timeScale/2,numUnitsInTick});
				}
				int nalHrdParametersPresentFlag=bitStream.getBit();
				if (nalHrdParametersPresentFlag==1) {
					parseHrdParameters(bitStream);
				}
				int vclHrdParametersPresentFlag=bitStream.getBit();
				if (vclHrdParametersPresentFlag==1) {
					parseHrdParameters(bitStream);
				}
				cpbDpbDelaysPresentFlag= (nalHrdParametersPresentFlag==1 || vclHrdParametersPresentFlag==1);
			}
		}
		
	}
	private void parseScalingList(BitStream bitStream, int scalingListSize) {
	    //From http://mpeg4ip.cvs.sourceforge.net/mpeg4ip/mpeg4ip/util/h264/main.cpp?revision=1.17&view=markup
	    int lastScale=8, nextScale=8;
	    for (int pos=0; pos<scalingListSize; pos++)
	    {
	        if (nextScale!=0)
	        {
	            int deltaScale=bitStream.getExpGolombCode();
	            nextScale=(lastScale+deltaScale+256)%256;
	        }
	        lastScale=nextScale;
	    }	
	}
	private void parseHrdParameters(BitStream bitStream) {
		int cpbCntMinus1=bitStream.getExpGolombCode();
		byte scale=bitStream.getByte();//bit_rate_scale & cpb_size_scale;
		for (int i=0;i<=cpbCntMinus1;i++) {
			int bit_rate_value_minus1=bitStream.getExpGolombCode();
			int cpb_size_value_minus1=bitStream.getExpGolombCode();
			int cbr_flag=bitStream.getBit();
		}
		int initial_cpb_removal_delay_length_minus1=bitStream.getBits(5);
		cpbRemovalDelayLengthMinus1=bitStream.getBits(5);
		dpbOutputDelayLengthMinus1=bitStream.getBits(5);
		int time_offset_length=bitStream.getBits(5);
	}

}
