package common.mpeg;

import java.io.IOException;
import java.io.InputStream;

import common.MuxBuffer;
import common.Util;

public class MpegPacketSourceBuffer extends MuxBuffer {
	private PesPacketInfo info;
	
	public MpegPacketSourceBuffer() {
		this(0);
	}
	
	public MpegPacketSourceBuffer(int bytes) {
		super(bytes);
	}
	
	public int findStartCode(byte type, int offset) {
		return MpegUtil.findStartCode(type, buffer, offset);
	}
	public int findNextStartCode(int offset) {
		return MpegUtil.findNextStartCode(buffer, offset);
	}
	public byte getType(int start) {
		return buffer.get(start + 3);
	}
	public int getPacketSize(int start) {
		if (buffer.limit() >= start+6) {
			return get16(start + 4) + 6;
		} else {
			return -1;
		}
	}
	public int getPaddingEnd(int start) {
		while (buffer.get(start)==MpegConstants.PADDING) {
			start++;
		}
		return start;
	}
	public int getPesHeaderSize(int offset) {
		return MpegUtil.getPesHeaderSize(buffer, offset);
	}
	

	/*public final int read(InputStream in, int len) throws IOException {
		ensureCapacity(len);
		int bytes=in.read(buffer, 0, len);
		if (bytes > 0) {
			this.len=bytes;
		}
		return bytes;
	}
	
	public final int append(InputStream in, int len) throws IOException {
		//Should verfiy buffer size
		int bytes=in.read(buffer, this.len, len);
		if (bytes > 0) {
			this.len+=bytes;
		}
		return bytes;
	}
	public void readRemainingHeader(InputStream in) throws IOException {
		if (len < 9) {
			append(in, 9-len);
		}
		append(in, MpegUtil.getHeaderBytes(buffer, 0));
	}*/
	public void clear() {
		limit(0);
		info=null;
	}
	
	public PesPacketInfo getPesPacket() {
		if (info==null) {
			info=MpegUtil.findPesPacket(buffer, 0);
		} else if (info.isComplete()) {
			//Start a new packet
			info=MpegUtil.findPesPacket(buffer, info.getEnd());
		} else {
			MpegUtil.findPesPacket(buffer, info);
		}
		return info; 
	}
	public void discard(int bytes) {
		super.discard(bytes);
		info=null;
	}
}
