package muxer;

import java.io.IOException;
import java.util.Iterator;

import common.Logger;
import common.PropertiesReader;
import common.StatusI;
import common.packet.Packet;

public class PacketBuffer extends MuxFilter {
	private Logger logger=Logger.getLogger(this.getClass());
	private static final int MIN_HEAD=100;
	private static final int MIN_FREE=1000;
	public static final PacketBufferRegulatorI NEVER_FULL= new PacketBufferRegulatorI() {
		public void addedPacket(PacketBuffer packetBuffer, Packet lastPacket) {}
		public int isFull() {
			return MuxManager.DO_NOTHING;
		}
		final public void removedPacket(Packet packet){};
	};
	private class PacketBufferIterator implements Iterator {
		private PacketBufferIterator() {  
			if (count > 0) {
				index=firstPacketIndex;
			} else {
				index = lastPacketIndex+1;
			}
		}
		private int index;
		
		public boolean hasNext() {
			return (index <=lastPacketIndex);
		}
		public void remove() {
			throw new UnsupportedOperationException();
		}
		public Object next() {
			while (lastPacketIndex >= index) {
				if (packets[index] != null) {
					return packets[index++];
				} else {
					index++;
				}
			}
			//Shouldn't happen
			return null;
		}
	}
	//private TreeMap packetMap=new TreeMap();
	private Packet packets[]=new Packet[PropertiesReader.getInstance().getIntProperty("muxer.PacketBuffer.bufferPackets", 8000)];
	private int indexOffset=-1;
	private int count=0;
	private int firstPacketIndex=-1;
	private int lastPacketIndex=-1;
	
	//private PacketBufferRegulatorI regulator; 
	private PacketBufferRegulatorI regulators[]=new PacketBufferRegulatorI[3];
	private int regulatorIndex=0;
	
	public PacketBuffer() {};
	
	public PacketBuffer(PacketBufferRegulatorI regulator) {
		addRegulator(regulator);
	}
	private int popPacket() throws IOException {
		Packet packet=packets[firstPacketIndex];
		packets[firstPacketIndex++]=null;
		count-=1;
		if (count==0) {
			reset();
		} else {
			while (packets[firstPacketIndex]==null) {
				firstPacketIndex++;
			}
		}
		//Packet packet=(Packet)packetMap.remove(packetMap.firstKey());
		for (int i=0;i<regulatorIndex;i++) {
			regulators[i].removedPacket(packet);
		}
		return super.addPacket(packet);
	};
	private int isFull() {
		for (int i=0;i<regulatorIndex;i++) {
			int rc=regulators[i].isFull();
			if (rc !=StatusI.CONTINUE) {
				return rc;
			}
		}
		return StatusI.CONTINUE;
	}
	public int addPacket(Packet packet) throws IOException {
		int index;
		if (count > 0) {
			index=packet.getIndex() - indexOffset;
			if (index >= packets.length) {
				pack();
				index=packet.getIndex() - indexOffset;
			}
			if (index >lastPacketIndex) {
				lastPacketIndex=index;
			} else if (index < firstPacketIndex) {
				firstPacketIndex=index;
			}
		} else {
			index=packet.getIndex();
			if (index>=packets.length) {
				index=firstPacketIndex=lastPacketIndex=0;
				indexOffset=packet.getIndex();
			} else {
				indexOffset=0;
				firstPacketIndex=lastPacketIndex=index;
			}
		}
		packets[index]=packet;
		count++;
		if (count > (lastPacketIndex-firstPacketIndex + 1)) {
			System.out.println("Doah!");
		}
		//Inform the regulators of a new packet
		for (int i=0;i<regulatorIndex;i++) {
			regulators[i].addedPacket(this, packet);
		}
		//packetMap.put(Integer.valueOf(packet.getIndex()),packet);
		/*switch (isFull()) {
			case MuxManager.CONTINUE:
				return popPacket();
			case MuxManager.ABORT:
				return MuxManager.ABORT;
			default:
				return MuxManager.CONTINUE;
		}*/
		int rc;
		while ((rc=isFull())==MuxManager.CONTINUE) {
			rc=popPacket();
			if (rc!=MuxManager.CONTINUE) {
				break;
			}
		} 
		if (rc==MuxManager.ABORT) {
			return rc;
		}
		return MuxManager.CONTINUE;
	}
	
	public boolean handlesRaw(int container) {
		return true;
	}
	public void streamEnd() throws IOException {
		int rc=MuxManager.CONTINUE;
		while (count !=0 && rc==MuxManager.CONTINUE) {
			try {
				rc=popPacket();
			} catch (IOException e) {
			}
		}
		super.streamEnd();
	}
	
	public void addRegulator(PacketBufferRegulatorI regulator) {
		regulators[regulatorIndex++]=regulator;
	}
	public void removeRegulator(PacketBufferRegulatorI regulator) {
		int offset=0;
		for (int i=0;i<regulatorIndex;i++) {
			if (regulator.equals(regulators[i])) {
				offset=-1;
			} else {
				regulators[i+offset]=regulators[i];
			}
		}
		regulatorIndex +=offset;
	}
	/*public void setRegulator(PacketBufferRegulatorI regulator) {
		this.regulator = regulator;
	}*/
	public Packet getFirstPacket(int streamId) {
		for (int i=firstPacketIndex;i<lastPacketIndex;i++) {
			if (packets[i] != null && packets[i].getStreamId()==streamId) {
				return packets[i];
			}
		}
		return null;
	}
	public Packet getPacket(int index) {
		return packets[index-indexOffset];
	}
	public int getLastIndex() {
		return lastPacketIndex-indexOffset;
	}
	public Iterator<Packet> iterator() {
		return new PacketBufferIterator();
	}
	public void reset() {
		count=0;
		firstPacketIndex=lastPacketIndex=indexOffset=-1;
	}
	public int size() {
		return count;
	}
	public int getPosition() {
		return POSITION_BUFFER;
	}
	private void pack() {
		if (logger.isTrace()) {
			float util=((float)count)/(float)(lastPacketIndex-firstPacketIndex + 1);
			logger.trace("Packing buffer.  Utilization: " + util /*+ ": " + count + ": " + firstPacketIndex + "-" + lastPacketIndex*/);
		}
		int len=lastPacketIndex - firstPacketIndex + 1;
		
		System.arraycopy(packets, firstPacketIndex, packets, MIN_HEAD, len);
		lastPacketIndex-=firstPacketIndex-MIN_HEAD;
		indexOffset+=firstPacketIndex-MIN_HEAD;
		firstPacketIndex=MIN_HEAD;

		if (packets.length-lastPacketIndex >= MIN_FREE) {
			for (int i=lastPacketIndex+1;i<packets.length;i++) {
				packets[i]=null;
			}
		} else { 
			Packet newPackets[]=new Packet[packets.length+MIN_FREE];
			if (logger.isDebug()) {
				for (int i=0;i<regulatorIndex;i++) {
					logger.debug(regulators[i]);
				}
				logger.debug("Packet buffer growing to " + newPackets.length);
			}
			System.arraycopy(packets, 0, newPackets, 0, lastPacketIndex+1);
			packets=newPackets;
		}
	}
}
