package regulator;

import java.util.Iterator;

import muxer.MuxManager;
import muxer.PacketBuffer;
import muxer.PacketBufferRegulatorI;

import common.Logger;
import common.packet.ClockReferenceI;
import common.packet.Packet;

public class StreamPtsPacketCountRegulator implements PacketBufferRegulatorI {
	private static final int CONTROL_FACTOR=2000;
	private static final int STREAM_ID=0;
	private static final int COUNT=1;
	private int[][] streams;
	private int fullCount;
	private boolean full=false;
	private Logger logger=Logger.getLogger(this.getClass());

	public  StreamPtsPacketCountRegulator(int streamIds[], int fullCount) {
		streams=new int[streamIds.length][2];
		for (int i=0;i<streamIds.length;i++) {
			this.streams[i][STREAM_ID]=streamIds[i];
			
		}
		this.fullCount=fullCount;
	}
	
	public void sync(PacketBuffer packetBuffer) {
		for (int i=0;i<streams.length;i++) {
			streams[i][COUNT]=0;
		}
		for (Iterator<Packet> i=packetBuffer.iterator();i.hasNext();) {
			Packet packet=(Packet)i.next();
			int stream[]=getStream(packet.getStreamId());
			if (stream !=null && packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS) >= 0) {
				stream[COUNT]++;
			}
		}
		checkFull();
	}
	private void checkFull() {
		full=false;
		for (int i=0;i<streams.length && !full;i++) {
			full|=streams[i][COUNT]>=fullCount;
		}
		
	}
	private int[] getStream(final int streamId) {
		for (int i=0;i<streams.length;i++) {
			if (streams[i][STREAM_ID]==streamId) {
				return streams[i];
			}
		}
		return null;
	}
	public void addedPacket(PacketBuffer packetBuffer, Packet lastPacket) {
		int stream[]=getStream(lastPacket.getStreamId());
		if (stream !=null && lastPacket.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS) >= 0) {
			stream[COUNT]++;
			if (!full) {
				checkFull();
			}
		}
	}
	public int isFull() {
		if (full) {
			//System.out.println(packetBuffer.size());
			return MuxManager.CONTINUE;
		}
/*		if (packetBuffer.size() > fullCount*CONTROL_FACTOR) {
			logger.info("Before sync: " + toString());
			sync(packetBuffer);
			logger.info("After sync: " + toString());
			throw new RuntimeException("Possible corrupt PMT, no packet in streams in "+fullCount*CONTROL_FACTOR+" packets");
		}*/
		return MuxManager.DO_NOTHING;
	}
	public String toString() {
		StringBuffer sb=new StringBuffer();
		String sep="";
		sb.append('{');
		for (int i=0;i<streams.length;i++) {
			sb.append(sep);
			sep=", ";
			sb.append('[');
			sb.append(Integer.toString(streams[i][STREAM_ID], 16));
			sb.append(',');
			sb.append(streams[i][COUNT]);
			sb.append(']');
		}
		sb.append('}');
		return sb.toString();
	}
	public void removedPacket(Packet packet) {
		int stream[]=getStream(packet.getStreamId());
		if (stream !=null && packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS) >=0) {
			stream[COUNT]--;
			checkFull();
		}
	}
	/**
	 * @return the streamId with the most packets
	 */
	public int getLargestStream() {
		int streamId=streams[0][STREAM_ID];
		int streamCount=streams[0][COUNT];
		for (int i=1;i<streams.length;i++) {
			if (streams[i][COUNT] > streamCount) {
				streamCount=streams[i][COUNT];
				streamId=streams[i][STREAM_ID];
			}
		}
		return streamId;
	}
}
