package ui.tonmt;

import io.NameTypeFileFilter;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class ToNmtPreferenceDialog extends JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = 4080708020244321764L;
	public static final String CODEC_ACTION_CODE="Core";
	public static final String CODEC_ACTION_TRANSCODE="Transcode";
	public static final String CODEC_ACTION_PASSTHROUGH="Passthrough";

	
	private static final String[] CONVERSION_CODECS={ToNMTPrefs.AUDIO_TYPE_AC3, ToNMTPrefs.AUDIO_TYPE_DTS, ToNMTPrefs.AUDIO_TYPE_LPCM};
	private static final String[] COMPLETED_ACTIONS={ToNMTPrefs.COMPLETED_ACTION_NONE, ToNMTPrefs.COMPLETED_ACTION_EXIT, ToNMTPrefs.COMPLETED_ACTION_PROCESS};
	
	
	private class PathChooser implements ActionListener {
		protected JFileChooser pathChooser=new JFileChooser();
		protected JTextField textField;
		
		private PathChooser() {
			pathChooser.setMultiSelectionEnabled(false);
			pathChooser.setDialogType(JFileChooser.OPEN_DIALOG);
			pathChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}
		
		public void actionPerformed(ActionEvent arg0) {
			File file=new File(textField.getText());
			if (!file.isDirectory()) {
				file=file.getParentFile();
			}
			pathChooser.setCurrentDirectory(file);
			int returnVal = pathChooser.showOpenDialog(ToNmtPreferenceDialog.this);
			if (returnVal==JFileChooser.APPROVE_OPTION) {
				textField.setText(pathChooser.getSelectedFile().getAbsolutePath());
			}
		}
		private void setTextField(JTextField textField) {
			this.textField=textField;
		}
	}
	
	private class ExeChooser extends PathChooser {
		private ExeChooser() {
			pathChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			pathChooser.addChoosableFileFilter(new NameTypeFileFilter(new String[]{".exe"}, "Executable"));
			pathChooser.setAcceptAllFileFilterUsed(false);
		}
	}

	public ToNmtPreferenceDialog() {
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.setSize(500,515);
		this.setTitle("Preferences");
		Container codecPanel=this.getContentPane();
		codecPanel.setLayout(new GridBagLayout());
		//JPanel codecPanel=new JPanel();
		//this.getRootPane().add(codecPanel);
		codecPanel.setLayout(new GridBagLayout());
		GridBagConstraints gbc1 = new GridBagConstraints();
		gbc1.gridx=gbc1.gridy=0;
		gbc1.anchor=GridBagConstraints.WEST;
		gbc1.fill=GridBagConstraints.HORIZONTAL;
		gbc1.weightx=2;
		gbc1.gridwidth=4;
		
		final ToNMTPrefs prefs=ToNMTPrefs.getPrefs();
		
		final JTextField tempDirPathField=addFileSelector(codecPanel, "Temporary Directory:", prefs.getDemuxPathName(), new PathChooser(), gbc1);
				
		codecPanel.add(new JLabel("Conversion Output:"),gbc1);
		gbc1.gridy++;
		
		final JComboBox conversionComboBox=new JComboBox(CONVERSION_CODECS);
		codecPanel.add(conversionComboBox,gbc1);
		gbc1.gridy++;
		conversionComboBox.setSelectedItem(prefs.getConvertToType());
		
		codecPanel.add(new JLabel(" "), gbc1);
		gbc1.gridy++;
		
		codecPanel.add(new JLabel("HD DVD Options:"), gbc1);
		gbc1.gridy++;
		
		JPanel trueHDPanel=new JPanel();
		codecPanel.add(trueHDPanel, gbc1);
		gbc1.gridy++;
		trueHDPanel.setLayout(new FlowLayout(FlowLayout.LEFT,2,0));
		//This is why I HATE Swing
		Dimension labelDimension=new Dimension(50,10);
		JLabel trueHDLabel=new JLabel("TrueHD:");
		trueHDLabel.setPreferredSize(labelDimension);
		trueHDPanel.add(trueHDLabel);
		final ButtonGroup trueHDButtonGroup=addRadioButtonGroup(new String[][]{{"Convert","c"},/*{"Passthrough","p"},*/{"Passthrough + AC3","ac3"}},prefs.getTrueHDAction(),trueHDPanel);
		
		JPanel eac3Panel=new JPanel();
		codecPanel.add(eac3Panel, gbc1);
		gbc1.gridy++;
		eac3Panel.setLayout(new FlowLayout(FlowLayout.LEFT,2,0));
		JLabel eac3Label=new JLabel("E-AC3:");
		eac3Label.setPreferredSize(labelDimension);
		eac3Panel.add(eac3Label);
		final ButtonGroup eac3ButtonGroup=addRadioButtonGroup(new String[][]{{"Convert","c"},{"Passthrough","p"}},prefs.getEAC3Action(),eac3Panel);
		
		JPanel dtsPanel=new JPanel();
		codecPanel.add(dtsPanel, gbc1);
		gbc1.gridy++;
		dtsPanel.setLayout(new FlowLayout(FlowLayout.LEFT,2,0));
		JLabel dtsLabel=new JLabel("DTS:");
		dtsLabel.setPreferredSize(labelDimension);
		dtsPanel.add(dtsLabel);
		final ButtonGroup dtsButtonGroup=addRadioButtonGroup(new String[][]{{"Convert","c"},{"Passthrough","p"}},prefs.getDtsAction(),dtsPanel);

		final JCheckBox sdCheckBox=new JCheckBox("Ensure SD Audio",prefs.isEnsureSDAudio());
		codecPanel.add(sdCheckBox, gbc1);
		gbc1.gridy++;
		
		final JCheckBox dialNormCheckBox=new JCheckBox("Keep Dialog Normalization",prefs.isKeepDialogNormalization());
		codecPanel.add(dialNormCheckBox, gbc1);
		gbc1.gridy++;
		
		codecPanel.add(new JLabel("BluRay Options:"), gbc1);
		gbc1.gridy++;

		final JCheckBox coreCheckBox=new JCheckBox("Core dts-HD & TrueHD/AC3",prefs.isCore());
		codecPanel.add(coreCheckBox, gbc1);
		gbc1.gridy++;

		codecPanel.add(new JLabel("Completed Action:"), gbc1);
		gbc1.gridy++;
		
		final JComboBox completedComboBox=new JComboBox(COMPLETED_ACTIONS);
		codecPanel.add(completedComboBox,gbc1);
		gbc1.gridy++;
		
		final JTextField processText=new JTextField(prefs.getCompletedProcess());
		codecPanel.add(processText,gbc1);
		gbc1.gridy++;
		
		final ActionListener completeListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				processText.setEnabled(ToNMTPrefs.COMPLETED_ACTION_PROCESS.equals(completedComboBox.getSelectedItem()));
			}
		};
		completedComboBox.addActionListener(completeListener);
		completedComboBox.setSelectedItem(prefs.getCompletedAction());
	
		codecPanel.add(new JLabel(" "), gbc1);
		gbc1.gridy++;

		final JTextField eac3toPathField=addFileSelector(codecPanel, "EAC3To Home:", prefs.getEac3toPath(), new ExeChooser(), gbc1);
		final JTextField tsMuxerPathField=addFileSelector(codecPanel, "TsMuxer Home:", prefs.getTsMuxerPath(), new ExeChooser(), gbc1);
		
		codecPanel.add(new JLabel(" "), gbc1);
		gbc1.gridy++;
		
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.gridy=gbc1.gridy+1;
		gbc.gridx=2;
		gbc.anchor=GridBagConstraints.EAST;
		JButton cancelButton=new JButton(" Cancel ");
		codecPanel.add(cancelButton, gbc);
		gbc.gridx++;
		gbc.anchor=GridBagConstraints.CENTER;
		JButton okButton=new JButton("    OK    ");
		codecPanel.add(okButton, gbc);
		
		final ActionListener okListener=new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				String conversion=conversionComboBox.getSelectedItem().toString();
				if (ToNMTPrefs.AUDIO_TYPE_DTS.equals(conversion)) {
					JOptionPane.showMessageDialog(ToNmtPreferenceDialog.this, "dts conversion requires the SurCode dts encoder which is not provided in the default installation", "SurCode dts Warning", JOptionPane.WARNING_MESSAGE);
				}
				prefs.setDemuxPathName(tempDirPathField.getText());
				prefs.setConvertToType(conversion);
				prefs.setEac3toPath(eac3toPathField.getText());
				prefs.setTsMuxerPath(tsMuxerPathField.getText());
				prefs.setTrueHDAction(trueHDButtonGroup.getSelection().getActionCommand());
				prefs.setEAC3Action(eac3ButtonGroup.getSelection().getActionCommand());
				prefs.setDtsAction(dtsButtonGroup.getSelection().getActionCommand());
				prefs.setEnsureSDAudio(sdCheckBox.isSelected());
				prefs.setKeepDialogNormalization(dialNormCheckBox.isSelected());
				prefs.setCore(coreCheckBox.isSelected());
				prefs.setCompletedAction((String)completedComboBox.getSelectedItem());
				prefs.setCompletedProcess(processText.getText());
				
				ToNmtPreferenceDialog.this.dispose();
			}
		};
		okButton.addActionListener(okListener);
		final ActionListener cancelListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ToNmtPreferenceDialog.this.dispose();
			}
		};
		cancelButton.addActionListener(cancelListener);

	}
	
	public static void main(String args[]) {
		new ToNmtPreferenceDialog().show();
	}
	
	private ButtonGroup addRadioButtonGroup(String values[][], String defaultValue, JPanel panel) {
		ButtonGroup buttonGroup=new ButtonGroup();
		
		for (int i=0;i<values.length;i++) {
			JRadioButton radioButton=new JRadioButton(values[i][0], defaultValue.equals(values[i][1]));
			radioButton.setActionCommand(values[i][1]);
			panel.add(radioButton);
			buttonGroup.add(radioButton);
		}
		
		return buttonGroup;
	}
	
	private JTextField addFileSelector(Container codecPanel, String labelText, String value, PathChooser listener, GridBagConstraints gbc1) {
		codecPanel.add(new JLabel(labelText), gbc1);
		gbc1.gridy++;
		
		gbc1.gridwidth=3;
		JTextField textField=new JTextField(value);
		textField.setEditable(false);
		codecPanel.add(textField,gbc1);
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridy = gbc1.gridy;
		gbc.gridx=3;
		gbc.weightx=1;
		gbc.anchor=GridBagConstraints.CENTER;
		gbc.fill=GridBagConstraints.NONE;
		
		JButton button=new JButton("Browse...");
		codecPanel.add(button,gbc);		
		if (listener != null) {
			button.addActionListener(listener);
		}
		gbc1.gridy++;
		listener.setTextField(textField);
		return textField;
	}
}
